ProbBin.flowcytest <- function(controldata, stimuldata, N=100, varname="",
                               AnalyType=c("combined", "by.control"),
                               title="", MY.DEBUG=FALSE, PBobj.plotted=TRUE,
                               plots.made=c("both", "stimulated", "unstimulated"),...){

  ##  INPUT
  ## controldata  vector of values for the control data
  ## stimuldata   vector of values for the stimulated data
  ## varname      name of the variable
  ## AnalyType    Probability Binning either "by.control" or with
  #                    the "combined" (control & stimulated) data
  ## N            Number in each bin on the data specified in AnalyType
  ## title        title of the plot
  ## MY.DEBUG     boolean; if True, print statements of processes are printed

  ## OUTPUT
  #  Probability Binning Statistics and p-values are output
  # see summary.ProbBin.FCS
  
  AnalyType <- match.arg(AnalyType)
  controldata <- unlist(controldata)
  stimuldata <- unlist(stimuldata)
  cat("FLOWCYTEST: BAGGERLY & ROEDERER STATS", "\n\n")
  cat("   Number of observations in each bin:", N, "\n")
  cat("   Dataset used for Probability Binning:", AnalyType, "\n")
  PBobj<- ProbBin.FCS(controldata,
                           stimuldata,
                           N, varname=varname,
                           PBspec=AnalyType,
                           MY.DEBUG=MY.DEBUG)

  output <- summary(PBobj)
 
  yht <- PBobj$N.in.bin + 100
  if (PBobj.plotted==TRUE){
    plots.made <- match.arg(plots.made)
    plot(PBobj, varname=varname,
         title=paste(title,
           " *Binning Dataset=",
           AnalyType, " *N.in.bin=",
           N, sep=""),  yht=yht, plots.made=plots.made,...)
  }
  cat("\n\n\n")                               
  return(output)
}


