VRC.HVTNFCS<- function(myFCSobj,
                       gate1.vars=c(1,2),
                       gate2.vars=c(7,5),
                       gate3.vars=c(5,3),
                       MY.DEBUG=FALSE) {
  
  
  ## This function uses igate.FCS and subset.FCS to obtain the correct
  ## gated dataset for further analysis (using runtests.FCS)

  ## gate1:bipcut: Forward Scatter VS Side Scatter  (want the lymphocytes--central cluster) 
  ## gate2:bidcut: cd3 VS cd4 (want the cd3+) 
  ## gate3:biscut: cd4 vs cd8 (want cd4+ cd8- and cd4-cd8+, separately) (all three are
  ##                separate indices)

  ## will output the augmented data matrix of the FCS object with the indices of the sequential gates

  ## to subset, only the last gate can be subseted


  ###  may want to use 

  ## gate 1:
  cat("\n")
  cat("Gate 1-bipcut: FSC vs SSC: Select lymphocytes (the central polygon)", "\n")
  cat("\n")
  cat("\n")
   myFCSobj <- icreateGate(myFCSobj,
                    type="bidcut",
                    varpos=gate1.vars,
                     comment="VRC.HVTNFCS:gate1:Lymphocytes",
                    MY.DEBUG=MY.DEBUG)
  gatepos.1 <- dim(myFCSobj@gate)[2]
 
  ## gate 2:
  cat("\n")
  cat("Gate 2-bidcut: cd3 vs cd4: Cut the graph in half using 2 vertical lines (x min, x max) and 2 horizontal lines (y min,y max).", "\n")
  cat("   Select cd3 positive cells (Input upper bound of x & y = 1023 and lowerbound of y = 0)", "\n")
  cat("\n")
  cat("\n")
   myFCSobj <- icreateGate(myFCSobj, type="bidcut",
                    biscut.quadrant=NULL,
                    varpos=gate2.vars,
                    prev.gateNum=gatepos.1,
                    prev.IndexValue.In=1,
                    comment="VRC.HVTNFCS:gate2:cd3+",
                    MY.DEBUG=MY.DEBUG)
  gatepos.2 <- dim(myFCSobj@gate)[2]

 
  ## gate 3:

  cat("\n")
  cat("Gate 3.1-biscut: Cut the graph into 4 quadrants/quarters using a x (horizontal)-line, y (vertical)-line:", "\n")
  cat("   Select cd4+/cd8-  and cd4-/cd8+ quadrant cells", "\n")
  
  cat("\n")
  cat("   Selected the quadrant of cd4 positive cd8 negative cells (biscut.quadrant=+/-)", "\n")
  cat("\n")
  myFCSobj <- icreateGate(myFCSobj,
                    type="biscut",
                    biscut.quadrant="+/-",
                    varpos=gate3.vars,
                    prev.gateNum=gatepos.2,
                    comment="VRC.HVTNFCS:gate3.1:cd4+/cd8-",
                    prev.IndexValue.In=1,
                    MY.DEBUG=MY.DEBUG)

 
  cat("\n")
  cat("Gate 3.2-biscut: Cut the graph into 4 quadrants/quarters using a x (horizontal)-line, y (vertical)-line:", "\n")
  cat("   Selected the quadrant of cd4 negative cd8 positive cells (biscut.quadrant=-/+)", "\n")
 
  cat("\n")
  cat("Please wait...", "\n\n")
  
  gatepos.3 <- dim(myFCSobj@gate)[2] 
  gatingrange <- extractGateHistory(myFCSobj, gateNum=gatepos.3)$gatingrange
  myFCSobj <- createGate(myFCSobj,
                         varpos=gate3.vars, 
                         gatingrange=gatingrange,
                         type="biscut",
                         biscut.quadrant="-/+",
                         prev.gateNum=gatepos.2,
                         prev.IndexValue.In=1,
                         comment="VRC.HVTNFCS:gate3.2:cd4-/cd8+",
                         MY.DEBUG=MY.DEBUG)
  gatepos.4 <- dim(myFCSobj@gate)[2]       
  showgate.FCS(myFCSobj@data[,gate3.vars],
               gatingrange=gatingrange,
               Index=myFCSobj@gate[myFCSobj@current.data.obs,gatepos.4],
               type="biscut",
               biscut.quadrant="-/+",
               IndexValue.In=1,
               coltype="white")

 
  return(myFCSobj)
  
}
