breakpoints.ProbBin <- function(data.var, N) {
  #######################################################################
  ## OBJECTIVE
  ##
  ## To implement define the break points in data.var in which there are
  ## N observations in each bin.
  #######################################################################
  ##  INPUT VARIABLES
  ##
  ## Name       Description
  ##
  ## data.var   the variable to be binned
  ##
  ## N          the number of cells in each bin of the data.var histogram
  ##
  #######################################################################
  ##  OUTPUT
  ##
  ##  Name            Description
  ## breaks       a vector of the endpoints of the bins
  ##              (Note: that the first break value is the min value and
  ##              the last break value  is 
  ##               the max of the variable you are looking at).
  ##
  ##
  ##
  ## NOTE: Please note that each bin in the histograms (in ProbBin.FCS) will be determined
  ## such that the endpoint is included (ie, for a<b, (a,b] is the bin interval.
  #######################################################################
  data.var <- as.numeric(data.var)
  N <- as.numeric(N)
  sorted.samp <- sort(data.var, decreasing=FALSE)
  tot <- length(sorted.samp)
  nbins<-ceiling(tot/N)
  if (nbins==1) {
    breaks <- unique(c(min(sorted.samp)-1,max(sorted.samp)))  #includes first-1 through last break values
  } else if (nbins > 1) {
    breaks <- unique(c(min(sorted.samp)-1,
                       sorted.samp[N*seq(1:(nbins-1))],
                       max(sorted.samp)))  #includes first-1 through last break values
  }
  return(breaks) #returning the first-1 to the last break values, such that (-1, 0] is the first interval of the breaks
}
