make.density <- function(x,
                         y,
                         status=NULL,
                         x.grid = seq(0, 1025, by=25),
                         y.grid = seq(0, 1025,by=25),
                         type.CSP=c("count.diff",
                           "p.hat",
                           "p.hat.norm",
                           "z.stat")) {
  ################
  # INPUT
  ################
  # x   a vector of the data values for the x-axis
  # y   a vector of the data values for the y-axis
  # x.grid   a vector of grid marks to allocate x
  # y.grid   a vector of grid marks to allocate y

  ############
  # Output
  ############
  # z   matrix of values with in an x-y grid
  # n.cells  number of total observation in z

  z.0 <- make.grid(x[status==0], y[status==0], x.grid=x.grid, y.grid=y.grid)$z
  z.1 <- make.grid(x[status==1], y[status==1], x.grid=x.grid, y.grid=y.grid)$z

  type.CSP <- match.arg(type.CSP)
  if (type.CSP=="count.diff"){
    z <- z.1-z.0
    
  }

  if (type.CSP=="z.stat") {
    p.hat <- z.1/(z.1+z.0)

 
    p.bar <- mean(p.hat, na.rm=TRUE)
    se.p <- sqrt(p.bar*(1-p.bar)/length(p.hat[!is.na(p.hat)]))
    z <- (p.hat-p.bar)/se.p
    ##z <- ifelse(is.na(z), 0, z)
    ## missing will be white cells in the image plot
  }
  
 if (type.CSP=="p.hat") {
    z <- z.1/(z.1+z.0)
    ## what to do about missing values?   
    ## z <- ifelse(is.na(z), 0, z)
  }

   if (type.CSP=="p.hat.norm") {
    p.hat <- z.1/(z.1+z.0)

 
    p.bar <- 0.5
    se.p <- sqrt(p.bar*(1-p.bar)/length(p.hat[!is.na(p.hat)]))
    z <- (p.hat-p.bar)/se.p
   ## z <- ifelse(is.na(z), 0, z)
    
  }

      
  


  
  return(list(z=z,type.CSP=type.CSP))
  
}
