require("methods")
## ==========================================================================
## show method for FCSmetadata
## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
setMethod("show",
          signature(object="FCSmetadata"),
          function(object) {
            
            if (object@original){
              obj.size <- object@size
              obj.npar <- object@nparam
              orig.flag <- "original"
            } else {
              obj.size <- object@fcsinfo[["RFACSadd>>$TOT"]]
              obj.npar <- object@fcsinfo[["RFACSadd>>$PAR"]]
              orig.flag <- "non-original"
            }

            cat("FACSmetadata for",orig.flag, "FCS object:", object@objectname,
                  "from original file",object@filename,"\n",
                  "with ",obj.size,"cells and",
                  obj.npar,"parameters.\n")
            invisible(object)
          })

## ==========================================================================
## print method for FCSmetadata
## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
setMethod("print",
          signature(x="FCSmetadata"),
          function(x,...) {
            show(x)## do stuff
          })

## ==========================================================================
## summary method for FCSmetadata
## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
setMethod("summary",
          signature(object="FCSmetadata"),
          function(object,...) {
            ##show(object)## do stuff
            str(object)
          })

## JYW: I put in the single brackets because the
## name of the slot is preserved in the output

## JYW: I also opted to put 

## NOTE ABOUT THE ORIGINAL FLAG:
## the original flag only suggests that the
## data has been changed, NOT the metadata
## the original flag allows the user to
## extract and change the correct metadata
## (ie, the RFACSadd>> variables are changed when
## the original is FALSE or the original variables
## are themselves changes when the original flag
## is false.


## if original==FALSE
## must include the following parameters in the metadata:
## RFACSadd>>$TOT (vector of number of rows)
## RFACSadd>>$PAR (vector of number of columns)
## RFACSadd>>$PnS (vector of longnames)
## RFACSadd>>$PnN (vector of shortnames)
## RFACSadd>>$PnR (vector of ranges)

## original flag is ONLY changed when the data is changed

## if only part of the numeric index is valid, then only part
## of the fcsinfo list is output with warnings

## ==========================================================================
## subsetting method for FCSmetadata
## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
setMethod("[",
          signature(x="FCSmetadata"),
          function(x,i,j,...,drop) {
            ## "i" can ONLY be EITHER a single character input
            ## OR a vector of numeric values
            ## returns only a single slot if i=character
            ## returns list elements in "fcsinfo" if i=numeric
            
            if ( !is.character(i) & !is.numeric(i)){
              ## i is neither numeric nor character
              stop("Input index is neither character nor numeric.")
            } else {
              if (is.character(i)){
                ## try to find it in the metadata ...
                ## JYW: I opted not to do the nested if statements
                ## because they were hard to read for me...
                
                ## are there any slotnames?
                
                ## are there any fcsinfo slotnames?
                if (length(i)>1){
                  stop("Only single entry for indexing by character slot name allowed.")
                }
                
                ##  original flag... needs work
                
                if ( sum(i %in% slotNames(x))==1 ){
                  ## need original checks for
                  ## size, nparam, longnames, shortnames
                  if (x@original==FALSE){
                    if ( !(i %in% c("size", "nparam",
                                    "longnames", "shortnames",
                                    "paramranges"))){
                      return(slot(x,i))
                    } else { ## depends on the false original flag
                      if (i == "size"){
                        return(x@fcsinfo[["RFACSadd>>$TOT"]])
                      }
                      if ( i == "nparam"){
                        return(x@fcsinfo[["RFACSadd>>$PAR"]])
                      }
                      if (i == "longnames"){
                        return(x@fcsinfo[["RFACSadd>>$PnS"]])
                      }
                      if (i=="shortnames"){
                        return(x@fcsinfo[["RFACSadd>>$PnN"]])
                      }
                      
                      if (i=="paramranges"){
                        return(x@fcsinfo[["RFACSadd>>$PnR"]])
                      } 
                      
                    }
                  } else { ## x@original is TRUE
                    return(slot(x, i))
                  }
                } else {

                  if ( sum(i %in% names(x@fcsinfo))==1) {
                    return(x@fcsinfo[[which(names(x@fcsinfo)==i)]])
                  } else {
                    if (sum(i %in% c("$PnS", 
                                     unlist(strsplit(paste("$P",
                                                           1:x@nparam,
                                                           "S", sep="", collapse=","),
                                                     split=","))))==length(i)){
                      pos.index <- c("$PnS",
                                     unlist(strsplit(paste("$P", 1:x@nparam,
                                                           "S", sep="", collapse=","),
                                                     split=","))) %in% i
                      if (x@original==TRUE){
                        if (pos.index[1]==TRUE){
                          
                          return(x@longnames)
                          
                        } else {
                          return(x@longnames[(pos.index[-1])])
                        }
                      } else { ## x@original == FALSE
                        if (pos.index[1]==TRUE){
                        
                          return(x@fcsinfo[["RFACSadd>>$PnS"]])
                          
                        } else {
                          return(x@fcsinfo[["RFACSadd>>$PnS"]][(pos.index[-1])])
                        }
                      }
                    
                    } else {
                      if (sum(i %in% c("$PnN", 
                                       unlist(strsplit(paste("$P",
                                                             1:x@nparam,
                                                             "N", sep="", collapse=","),
                                                       split=","))))==length(i)){
                        pos.index <- c("$PnN", unlist(strsplit(paste("$P", 1:x@nparam,
                                                                     "N", sep="", collapse=","),
                                                               split=","))) %in% i
                        if (x@original==TRUE){
                          if (pos.index[1]==TRUE){
                            return(x@shortnames)
                          } else {
                            return(x@shortnames[(pos.index[-1])])
                          }
                        } else{ ## if (x@original==FALSE)
                          if (pos.index[1]==TRUE){
                            return(x@fcsinfo[["RFACSadd>>$PnN"]])
                          } else {
                            return(x@fcsinfo[["RFACSadd>>$PnN"]][(pos.index[-1])])
                          }
                        }
                      } else {
                        if (sum(i %in% c("$PnR", 
                                         unlist(strsplit(paste("$P",
                                                               1:x@nparam,
                                                               "R", sep="", collapse=","),
                                                         split=","))))==length(i)){
                          pos.index <- c("$PnR", unlist(strsplit(paste("$P", 1:x@nparam,
                                                                       "R", sep="", collapse=","),
                                                                 split=","))) %in% i
                          if (x@original==TRUE){
                            if (pos.index[1]==TRUE){
                              return(x@paramranges)
                            } else {
                              return(x@paramranges[(pos.index[-1])])
                            }
                          } else {## if (x@original==FALSE)
                            if (pos.index[1]==TRUE){
                              return(x@fcsinfo[["RFACSadd>>$PnR"]])
                            } else {
                              return(x@fcsinfo[["RFACSadd>>$PnR"]][(pos.index[-1])])
                            }
                          }
                        } else {
                          if (!(i %in% c("$MODE", "$TOT", "$PAR"))){
                            warning("The Slot Name cannot be found in the metadata.")
                            return(NULL)
                          } else {
                            if (i=="$MODE"){
                              return(x@mode)
                            }
                            if (i=="$TOT"){
                              if (x@original==TRUE){
                                return(x@size)
                              } else {
                                return(x@fcsinfo[["RFACSadd>>$TOT"]])
                              }
                            }
                            if (i=="$PAR"){
                              if (x@original==TRUE){
                                return(x@nparam)
                              } else {
                                return(x@fcsinfo[["RFACSadd>>$PAR"]])
                              }
                            }
                          }
                        
                        
                        }
                      }
                    }
                  }
                }
              } ## is.character(i)

              if (is.numeric(i)){
                ## JYW: will do it the long way, without x@metadata[[i]]
                ## will only return value if all of the indices are found in the fcsinfo
                
                index.i <- i %in% 1:length(x@fcsinfo)
                
                if (sum(index.i==1) !=length(i)){
                  warning("Part or all of the Index cannot be found in the metadata.")
                  if (sum(index.i==1)==0){
                    return(NULL)
                  }
                }
                return(x@fcsinfo[i[index.i==1]])
              }
            }
              
          
          })


## ==========================================================================
## replace method for FCSmetadata
## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
## JYW: I put in the single brackets because the
## name of the slot is preserved in the output
setReplaceMethod("[",
          signature(x="FCSmetadata"),
                 function(x,i,j,...,value) {
                   ## "i" can ONLY be EITHER a single character input OR a vector of numeric values
                   ## returns only a single slot if i=character
                   ## returns list elements in "fcsinfo" if i=numeric
                   
                   if ( !is.character(i) & !is.numeric(i)){
                     ## i is neither numeric nor character
                     stop("Input index is neither character nor numeric.")
                   } else {
                     if (is.character(i)){
                       ## try to find it in the metadata ...
                       ## JYW: I opted not to do the nested if statements
                       ## because they were hard to read for me...
                       
                       ## are there any slotnames?
                       
                       ## are there any fcsinfo slotnames?
                       if (length(i)>1){
                         stop("Only single entry for indexing by character slot name allowed.")
                       }
                
                       ##  original flag... needs work
                
                       if ( sum(i %in% slotNames(x))==1 ){
                         ## need original checks for
                         ## size, nparam, longnames, shortnames
                         if (x@original==FALSE){
                           if ( !(i %in% c("size", "nparam",
                                           "longnames", "shortnames",
                                           "paramranges"))){
                             slot(x,i) <- value
                           } else { ## depends on the false original flag
                             if (i == "size"){
                               x@fcsinfo[["RFACSadd>>$TOT"]] <- value
                             }
                             if ( i == "nparam"){
                               x@fcsinfo[["RFACSadd>>$PAR"]] <- value
                             }
                             if (i == "longnames"){
                               x@fcsinfo[["RFACSadd>>$PnS"]] <- value
                             }
                             if (i=="shortnames"){
                               x@fcsinfo[["RFACSadd>>$PnN"]] <- value
                             }
                      
                             if (i=="paramranges"){
                               x@fcsinfo[["RFACSadd>>$PnR"]] <- value
                             } 
                             
                           }
                         } else { ## x@original is TRUE
                           slot(x, i) <- value
                         }
                       } else {

                         if ( sum(i %in% names(x@fcsinfo))==1) {
                           x@fcsinfo[[which(names(x@fcsinfo)==i)]] <- value
                         } else {
                           if (sum(i %in% c("$PnS", 
                                            unlist(strsplit(paste("$P",
                                                                  1:x@nparam,
                                                                  "S", sep="", collapse=","),
                                                            split=","))))==length(i)){
                             pos.index <- c("$PnS",
                                            unlist(strsplit(paste("$P", 1:x@nparam,
                                                                  "S", sep="", collapse=","),
                                                            split=","))) %in% i
                             if (x@original==TRUE){
                               if (pos.index[1]==TRUE){
                                 
                                 x@longnames <- value
                          
                               } else {
                                 x@longnames[(pos.index[-1])] <- value
                               }
                             } else { ## x@original == FALSE
                               if (pos.index[1]==TRUE){
                        
                                 x@fcsinfo[["RFACSadd>>$PnS"]] <- value
                          
                               } else {
                                 x@fcsinfo[["RFACSadd>>$PnS"]][(pos.index[-1])] <- value
                               }
                             }
                    
                           } else {
                             if (sum(i %in% c("$PnN", 
                                              unlist(strsplit(paste("$P",
                                                                    1:x@nparam,
                                                                    "N", sep="", collapse=","),
                                                              split=","))))==length(i)){
                               pos.index <- c("$PnN", unlist(strsplit(paste("$P", 1:x@nparam,
                                                                            "N", sep="", collapse=","),
                                                                      split=","))) %in% i
                               if (x@original==TRUE){
                                 if (pos.index[1]==TRUE){
                                   x@shortnames <- value
                                 } else {
                                   x@shortnames[(pos.index[-1])] <- value
                                 }
                               } else{ ## if (x@original==FALSE)
                                 if (pos.index[1]==TRUE){
                                   x@fcsinfo[["RFACSadd>>$PnN"]] <- value
                                 } else {
                                   x@fcsinfo[["RFACSadd>>$PnN"]][(pos.index[-1])] <- value
                                 }
                               }
                             } else {
                               if (sum(i %in% c("$PnR", 
                                                unlist(strsplit(paste("$P",
                                                                      1:x@nparam,
                                                                      "R", sep="", collapse=","),
                                                                split=","))))==length(i)){
                                 pos.index <- c("$PnR", unlist(strsplit(paste("$P", 1:x@nparam,
                                                                              "R", sep="", collapse=","),
                                                                        split=","))) %in% i
                                 if (x@original==TRUE){
                                   if (pos.index[1]==TRUE){
                                     x@paramranges <- value
                                   } else {
                                     x@paramranges[(pos.index[-1])] <- value
                                   }
                                 } else {## if (x@original==FALSE)
                                   if (pos.index[1]==TRUE){
                                     x@fcsinfo[["RFACSadd>>$PnR"]] <- value
                                   } else {
                                     x@fcsinfo[["RFACSadd>>$PnR"]][(pos.index[-1])] <- value
                                   }
                                 }
                               } else {
                                 if (!(i %in% c("$MODE", "$TOT", "$PAR"))){
                                   ## cannot be found, so we make a new slot
                                   len <- length(x@fcsinfo)
                                   x@fcsinfo[[len+1]] <- value
                                   ## we do not have the RFACSadd>> prefix when just
                                   ## changing/adding on to the metadata
                                   names(x@fcsinfo)[len+1] <- i ##paste("RFACSadd>>", i, sep="")
                       
                                  
                                 } else {
                                   if (i=="$MODE"){
                                     x@mode <- value
                                   }
                                   if (i=="$TOT"){
                                     if (x@original==TRUE){
                                       x@size <- value
                                     } else {
                                       x@fcsinfo[["RFACSadd>>$TOT"]] <- value
                                     }
                                   }
                                   if (i=="$PAR"){
                                     if (x@original==TRUE){
                                       x@nparam <- value
                                     } else {
                                       x@fcsinfo[["RFACSadd>>$PAR"]] <- value
                                     }
                                   }
                                 }
                                 
                                 
                               }
                             }
                           }
                         }
                       }
                     } ## is.character(i)

                     if (is.numeric(i)){
                       ## JYW: will do it the long way, without x@metadata[[i]]
                       ## will only return value if all of the indices are found in the fcsinfo
                       
                       index.i <- i %in% 1:length(x@fcsinfo)
                       
                       if (sum(index.i==1) !=length(i)){
                         warning("Part or all of the Index cannot be found in the metadata.")
                         
                       }
                       x@fcsinfo[i[index.i==1]] <- value
                     }
                   }
                   
                   
                   x
                 })


## JYW: I put in the single brackets because the
## name of the slot is preserved in the output

## ==========================================================================
## subsetting method for FCSmetadata
## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
## JYW: I also opted to put 
setMethod("[[",
          signature(x="FCSmetadata"),
          function(x,i,j,...,drop) {
            ## "i" can ONLY be EITHER a single character input OR a vector of numeric values
            ## returns only a single slot if i=character
            ## returns list elements in "fcsinfo" if i=numeric
            if ( !is.character(i) & !is.numeric(i)){
              ## i is neither numeric nor character
              stop("Input index is neither character nor numeric.")
            } else {
              if (is.character(i)){
                ## try to find it in the metadata ...
                ## JYW: I opted not to do the nested if statements
                ## because they were hard to read for me...
                
                ## are there any slotnames?
                
                ## are there any fcsinfo slotnames?
                if (length(i)>1){
                  stop("Only single entry for indexing by character slot name allowed.")
                }
                
                ##  original flag... needs work
                
                if ( sum(i %in% slotNames(x))==1 ){
                  ## need original checks for
                  ## size, nparam, longnames, shortnames
                  if (x@original==FALSE){
                    if ( !(i %in% c("size", "nparam",
                                    "longnames", "shortnames",
                                    "paramranges"))){
                      return(slot(x,i))
                    } else { ## depends on the false original flag
                      if (i == "size"){
                        return(x@fcsinfo[["RFACSadd>>$TOT"]])
                      }
                      if ( i == "nparam"){
                        return(x@fcsinfo[["RFACSadd>>$PAR"]])
                      }
                      if (i == "longnames"){
                        return(x@fcsinfo[["RFACSadd>>$PnS"]])
                      }
                      if (i=="shortnames"){
                        return(x@fcsinfo[["RFACSadd>>$PnN"]])
                      }
                      
                      if (i=="paramranges"){
                        return(x@fcsinfo[["RFACSadd>>$PnR"]])
                      } 
                      
                    }
                  } else { ## x@original is TRUE
                    return(slot(x, i))
                  }
                } else {

                  if ( sum(i %in% names(x@fcsinfo))==1) {
                    return(x@fcsinfo[[which(names(x@fcsinfo)==i)]])
                  } else {
                    if (sum(i %in% c("$PnS", 
                                     unlist(strsplit(paste("$P",
                                                           1:x@nparam,
                                                           "S", sep="", collapse=","),
                                                     split=","))))==length(i)){
                      pos.index <- c("$PnS",
                                     unlist(strsplit(paste("$P", 1:x@nparam,
                                                           "S", sep="", collapse=","),
                                                     split=","))) %in% i
                      if (x@original==TRUE){
                        if (pos.index[1]==TRUE){
                          
                          return(x@longnames)
                          
                        } else {
                          return(x@longnames[(pos.index[-1])])
                        }
                      } else { ## x@original == FALSE
                        if (pos.index[1]==TRUE){
                        
                          return(x@fcsinfo[["RFACSadd>>$PnS"]])
                          
                        } else {
                          return(x@fcsinfo[["RFACSadd>>$PnS"]][(pos.index[-1])])
                        }
                      }
                    
                    } else {
                      if (sum(i %in% c("$PnN", 
                                       unlist(strsplit(paste("$P",
                                                             1:x@nparam,
                                                             "N", sep="", collapse=","),
                                                       split=","))))==length(i)){
                        pos.index <- c("$PnN", unlist(strsplit(paste("$P", 1:x@nparam,
                                                                     "N", sep="", collapse=","),
                                                               split=","))) %in% i
                        if (x@original==TRUE){
                          if (pos.index[1]==TRUE){
                            return(x@shortnames)
                          } else {
                            return(x@shortnames[(pos.index[-1])])
                          }
                        } else{ ## if (x@original==FALSE)
                          if (pos.index[1]==TRUE){
                            return(x@fcsinfo[["RFACSadd>>$PnN"]])
                          } else {
                            return(x@fcsinfo[["RFACSadd>>$PnN"]][(pos.index[-1])])
                          }
                        }
                      } else {
                        if (sum(i %in% c("$PnR", 
                                         unlist(strsplit(paste("$P",
                                                               1:x@nparam,
                                                               "R", sep="", collapse=","),
                                                         split=","))))==length(i)){
                          pos.index <- c("$PnR", unlist(strsplit(paste("$P", 1:x@nparam,
                                                                       "R", sep="", collapse=","),
                                                                 split=","))) %in% i
                          if (x@original==TRUE){
                            if (pos.index[1]==TRUE){
                              return(x@paramranges)
                            } else {
                              return(x@paramranges[(pos.index[-1])])
                            }
                          } else {## if (x@original==FALSE)
                            if (pos.index[1]==TRUE){
                              return(x@fcsinfo[["RFACSadd>>$PnR"]])
                            } else {
                              return(x@fcsinfo[["RFACSadd>>$PnR"]][(pos.index[-1])])
                            }
                          }
                        } else {
                          if (!(i %in% c("$MODE", "$TOT", "$PAR"))){
                            warning("The Slot Name cannot be found in the metadata.")
                            return(NULL)
                          } else {
                            if (i=="$MODE"){
                              return(x@mode)
                            }
                            if (i=="$TOT"){
                              if (x@original==TRUE){
                                return(x@size)
                              } else {
                                return(x@fcsinfo[["RFACSadd>>$TOT"]])
                              }
                            }
                            if (i=="$PAR"){
                              if (x@original==TRUE){
                                return(x@nparam)
                              } else {
                                return(x@fcsinfo[["RFACSadd>>$PAR"]])
                              }
                            }
                          }
                        
                        
                        }
                      }
                    }
                  }
                }
              } ## is.character(i)

              if (is.numeric(i)){
                ## JYW: will do it the long way, without x@metadata[[i]]
                ## will only return value if all of the indices are found in the fcsinfo
                
                index.i <- i %in% 1:length(x@fcsinfo)
                
                if (sum(index.i==1) != length(i)){
                  warning("Part or all of the Index cannot be found in the metadata.")
                  if (sum(index.i==1)==0){
                    return(NULL)
                  }
                }
                return(x@fcsinfo[i[index.i==1]])
              }
            }
              
           
          })
           
### LIST OF KEY METADATA WORDS/slotnames
## NOTE: (single value of length 1 is returned unless otherwise noted in ())
## 1.  mode, $MODE
## 2.  size, $TOT
## 3.  nparam, $PAR
## 4.  shortnames (vector), $PnN (vector), $P1N, $P2N, ...
## 5.  longnames (vector), $PnS(vector), $P1S, $P2S, ...
## 6.  paramranges (vector), $PnR (vector), $P1R, $P2R
## 7.  filename
## 8.  objectname
## 9.  original
## 10. fcsinfo (includes RFACSadd>> when the data
##          is changed resulting in changes with  #2-6 (above))
## 11. any other name will be added as a new slot

## ==========================================================================
## replacement method for FCSmetadata
## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
setReplaceMethod("[[",
                 signature(x="FCSmetadata"),
                 function(x,i,j,...,value) {
                   ## "i" can ONLY be EITHER a single character input OR a vector of numeric values
                   ## returns only a single slot if i=character
                   ## returns list elements in "fcsinfo" if i=numeric
                   if ( !is.character(i) & !is.numeric(i)){
                     ## i is neither numeric nor character
                     stop("Input index is neither character nor numeric.")
                   } else {
                     if (is.character(i)){
                       ## try to find it in the metadata ...
                       ## JYW: I opted not to do the nested if statements
                       ## because they were hard to read for me...
                       
                       ## are there any slotnames?
                       
                       ## are there any fcsinfo slotnames?
                       if (length(i)>1){
                         stop("Only single entry for indexing by character slot name allowed.")
                       }
                
                       ##  original flag... needs work
                
                       if ( sum(i %in% slotNames(x))==1 ){
                         ## need original checks for
                         ## size, nparam, longnames, shortnames
                         if (x@original==FALSE){
                           if ( !(i %in% c("size", "nparam",
                                           "longnames", "shortnames",
                                           "paramranges"))){
                             slot(x,i) <- value
                           } else { ## depends on the false original flag
                             if (i == "size"){
                               x@fcsinfo[["RFACSadd>>$TOT"]] <- value
                             }
                             if ( i == "nparam"){
                               x@fcsinfo[["RFACSadd>>$PAR"]] <- value
                             }
                             if (i == "longnames"){
                               x@fcsinfo[["RFACSadd>>$PnS"]] <- value
                             }
                             if (i=="shortnames"){
                               x@fcsinfo[["RFACSadd>>$PnN"]] <- value
                             }
                      
                             if (i=="paramranges"){
                               x@fcsinfo[["RFACSadd>>$PnR"]] <- value
                             } 
                             
                           }
                         } else { ## x@original is TRUE
                           slot(x, i) <- value
                         }
                       } else {

                         if ( sum(i %in% names(x@fcsinfo))==1) {
                           x@fcsinfo[[which(names(x@fcsinfo)==i)]] <- value
                         } else {
                           if (sum(i %in% c("$PnS", 
                                            unlist(strsplit(paste("$P",
                                                                  1:x@nparam,
                                                                  "S", sep="", collapse=","),
                                                            split=","))))==length(i)){
                             pos.index <- c("$PnS",
                                            unlist(strsplit(paste("$P", 1:x@nparam,
                                                                  "S", sep="", collapse=","),
                                                            split=","))) %in% i
                             if (x@original==TRUE){
                               if (pos.index[1]==TRUE){
                                 
                                 x@longnames <- value
                          
                               } else {
                                 x@longnames[(pos.index[-1])] <- value
                               }
                             } else { ## x@original == FALSE
                               if (pos.index[1]==TRUE){
                        
                                 x@fcsinfo[["RFACSadd>>$PnS"]] <- value
                          
                               } else {
                                 x@fcsinfo[["RFACSadd>>$PnS"]][(pos.index[-1])] <- value
                               }
                             }
                    
                           } else {
                             if (sum(i %in% c("$PnN", 
                                              unlist(strsplit(paste("$P",
                                                                    1:x@nparam,
                                                                    "N", sep="", collapse=","),
                                                              split=","))))==length(i)){
                               pos.index <- c("$PnN", unlist(strsplit(paste("$P", 1:x@nparam,
                                                                            "N", sep="", collapse=","),
                                                                      split=","))) %in% i
                               if (x@original==TRUE){
                                 if (pos.index[1]==TRUE){
                                   x@shortnames <- value
                                 } else {
                                   x@shortnames[(pos.index[-1])] <- value
                                 }
                               } else{ ## if (x@original==FALSE)
                                 if (pos.index[1]==TRUE){
                                   x@fcsinfo[["RFACSadd>>$PnN"]] <- value
                                 } else {
                                   x@fcsinfo[["RFACSadd>>$PnN"]][(pos.index[-1])] <- value
                                 }
                               }
                             } else {
                               if (sum(i %in% c("$PnR", 
                                                unlist(strsplit(paste("$P",
                                                                      1:x@nparam,
                                                                      "R", sep="", collapse=","),
                                                                split=","))))==length(i)){
                                 pos.index <- c("$PnR", unlist(strsplit(paste("$P", 1:x@nparam,
                                                                              "R", sep="", collapse=","),
                                                                        split=","))) %in% i
                                 if (x@original==TRUE){
                                   if (pos.index[1]==TRUE){
                                     x@paramranges <- value
                                   } else {
                                     x@paramranges[(pos.index[-1])] <- value
                                   }
                                 } else {## if (x@original==FALSE)
                                   if (pos.index[1]==TRUE){
                                     x@fcsinfo[["RFACSadd>>$PnR"]] <- value
                                   } else {
                                     x@fcsinfo[["RFACSadd>>$PnR"]][(pos.index[-1])] <- value
                                   }
                                 }
                               } else {
                                 if (!(i %in% c("$MODE", "$TOT", "$PAR"))){
                                   ## cannot be found, so we make a new slot
                                   len <- length(x@fcsinfo)
                                   x@fcsinfo[[len+1]] <- value
                                   ## we do not have the RFACSadd>> prefix when just
                                   ## changing/adding on to the metadata
                                   names(x@fcsinfo)[len+1] <- i ##paste("RFACSadd>>", i, sep="")
                       
                                  
                                 } else {
                                   if (i=="$MODE"){
                                     x@mode <- value
                                   }
                                   if (i=="$TOT"){
                                     if (x@original==TRUE){
                                       x@size <- value
                                     } else {
                                       x@fcsinfo[["RFACSadd>>$TOT"]] <- value
                                     }
                                   }
                                   if (i=="$PAR"){
                                     if (x@original==TRUE){
                                       x@nparam <- value
                                     } else {
                                       x@fcsinfo[["RFACSadd>>$PAR"]] <- value
                                     }
                                   }
                                 }
                                 
                                 
                               }
                             }
                           }
                         }
                       }
                     } ## is.character(i)

                     if (is.numeric(i)){
                       ## JYW: will do it the long way, without x@metadata[[i]]
                       ## will only return value if all of the indices are found in the fcsinfo
                       
                       index.i <- i %in% 1:length(x@fcsinfo)
                       
                       if (sum(index.i==1) !=length(i)){
                         warning("Part or all of the Index cannot be found in the metadata.")
                         
                       }
                       x@fcsinfo[i[index.i==1]] <- value
                     }
                   }
                   
                   
                   x
                 
                 
                 })
## NOTE: [[<- for FCSmetdata
## can replace via numeric index with a value that is a list or vector
## ie.  ex.s4@metadata[[c(1,100000)]]<-c("g", "wrong")  or list("g", "wrong")

