"parallelCoordinates" <- function (x,
                                    varlabpos=1:dim(x)[2],
                                    variable.names=colnames(x),
                                    my.ylab ="",
                                    my.ylim =c(min(x), max(x)),
                                    at.y=seq(min(x), max(x),
                                      by=(max(x)-min(x))/20),
                                    each.ylab=at.y,
                                    scaled = FALSE,
                                    group= rep(1, dim(x)[1]),
                                    group.lty=group,
                                    group.col=group,
                                    group.lwd=group,
                                    superimpose=FALSE,...) {

  ##############################
  #  INPUT
  ##############################
  # x       matrix of the data (rows are the observations,
  #              columns are the variables)
  # scaled  boolean; if true then scale the data to the range 0,1
  #          centered about the minimum value
  # group

  ##############################
  # OUTPUT
  ##############################
  # plots each row observation across all column variables

  
  if (scaled == TRUE) {
   
    my.ylab <- paste(c("Scaled Values"), my.ylab, sep=" ")
    my.ylim <- c(0, 1)
    x <- apply(x, 2, function(col) {
      if ((max(col)-min(col)) != 0){
        (col - min(col))/(max(col) - min(col))
      } else {
        col
      }
    })
      #change the at.y if it is not scaled
  
    range.at.y <- range(at.y)
    if (abs(diff(range.at.y)) != 0){
      at.y <- at.y/(abs(diff(range.at.y)))
    }
   
  }
 
    #change each.ylab to at.y saled if
    #length(each.ylab)!=length(at.y)
 
  if (length(at.y)!=length(each.ylab)){
    each.ylab <- at.y
  }
  if (superimpose==FALSE){
    plot(0, 0, xlab = c("Variables"), xlim = c(1, max(varlabpos)), 
         ylab = my.ylab, ylim = my.ylim, type = "n", xaxt="n", yaxt="n",...)
    
    axis(2, at=at.y, lab=each.ylab)
    axis(1, at=varlabpos, lab=variable.names)
  }

  if (length(group) != dim(x)[1] || length(group.lty) != dim(x)[1]  || length(group.col) != dim(x)[1]  || length(group.lwd) != dim(x)[1]){ 
    warning(paste("The options group, group.lty, group.col, group.lwd should have a vector length of", dim(x)[1]))
  }
  apply(cbind(x,group.lty, group.col, group.lwd), 1, function(y) {
    lwd.pos <- length(y)
    col.pos <- length(y)-1
    lty.pos <- length(y)-2

    lines(varlabpos, y[-(lty.pos:lwd.pos)], lty=y[lty.pos], col=y[col.pos], lwd=y[lwd.pos])
  })
  legend(varlabpos[1], max(x), paste("group", unique(group)), lty=unique(group.lty), col=unique(group.col), lwd=unique(group.lwd))
}  
 
