plotdensity.FCS <- function(data,
                            varpos,
                            groups,
                            xlab,
                            ylab,
                            col,
                            xlim=NULL,
                            ylim=NULL,
                            main=NULL,
                            ...                        
                            ){
    ## Create a density plot for one parameter (e.g. FSC(varpos=1), SSC(varpos=2))    
    ## Nolwenn Le Meur
    ## 18 Sept 2005. Last revised 18 Feb 2007

    ## x            a cytoSet object extracted by the readCytoSet
    ## varpos       the variable column position of the data being examined
    ##              or a vector of column postions

   
    if(!is(data, "cytoSet") && !is(data[[1]],"FCS")){
        stop("input 'object' must be a cytoSet object or a list of FCS objects")
    }
    
    #extract values
    if(is(data,"cytoSet")){
        x <- vector("list")
        for (i in 1:length(data)){
            x[[i]]<-exprs(data[[i]])
        }
    }

    if(is(data[[1]],"FCS")){
        x <- vector("list")
        for (i in 1:length(data)){
            x<-lapply(data,fluors)
        }
    }
    
    
    if (missing(xlab)) xlab <- "Intensity"
    if (missing(ylab)) ylab <- "Density"
    if (!missing(col)) colors <-col
    if (missing(col)) colors <-c(1:length(x))
    if (is.null(xlim)) xlim <- range(x)
    if (!is.list(x)) x<- list(x)
    
    ##x <- lapply(x,function(z){z[,varpos]})
    ##lengthz <- lapply(x, function(z) length(z))
    ##data.toplot <- do.call(make.groups,x)
   
    ##grouping <- factor(rep(groups, lengthz),levels=c(groups),ordered=TRUE)
    ##data.toplot <- cbind(data.toplot,"groups"=grouping)
    
    ##densityplot(~data,groups=grouping,data=data.toplot,plot.points=FALSE,...)
    
    if(is.null(ylim)){
        xd <- density.lf(x[[1]][,varpos])
        plot(xd,type="n",xlab=xlab,ylab=ylab,xlim=xlim,main=main,...)
        for(i in 1:length(x)){
            xd <- density.lf(x[[i]][,varpos])
           lines(xd,col=colors[i],...)
        }
    }
    else{
        xd <- density.lf(x[[1]][,varpos])
        plot(xd,type="n",xlab=xlab,ylab=ylab,xlim=xlim,ylim=ylim,main=main,...)
        for(i in 1:length(x)){
            xd <- density.lf(x[[i]][,varpos])
            lines(xd,col=colors[i],...)
        }
    }
}
