\name{FHCRC.HVTNFCS}
\alias{FHCRC.HVTNFCS}
%- Also NEED an `\alias' for EACH other topic documented here.
\title{Fred Hutchinson Cancer Research Center Sequential Gating
  Procedure proposed by Julie McElrath's Lab}
\description{

  This function uses \code{\link{icreateGate}} and \code{\link{createGate}} to
  select the datapoints which are of particular interest.  The selection
  process is realized in an index column which is added to the data of
  the FCS object.  In particular, after a series of gating/datapoint
  selection sequences, the interferon gamma variable is of interest.

  To row reduce the data of the FCS object, the function,
  \code{\link{extractGatedData}} should be used on the last gate index to
  obtain the rows/cells and then should be used again to subset
  across columns to obtain the gamma interferon column.

}
\usage{

FHCRC.HVTNFCS(myFCSobj, gate1.vars = c(1, 2), gate2.vars = c(5, 7),
                 gate3.vars = c(3, 4),MY.DEBUG = FALSE)

}
%- maybe also `usage' for other objects documented here.
\arguments{
  \item{myFCSobj}{a FCS object}
  \item{gate1.vars}{The vector of column variable positions
    corresponding to Forward Scatter and Side Scatter variables for the
    first gate; default
    is column positions 1 and 2 respectively}
  \item{gate2.vars}{The vector of column variable positions
    corresponding to cd3 and cd8 variables for the
    second gate; default is column positions 5 and 7 respectively}
  \item{gate3.vars}{The vector of column variable positions
    corresponding to cd69 and Interferon Gamma variables for gate 3;
    default is column positions 3 and 4 respectively}
  \item{MY.DEBUG}{if TRUE, then will print the debugging statements;
    otherwise, if FALSE, then will surpress the debugging statements;
    default is FALSE}
}
\details{

  The Selection Sequence made by Julie McElrath's Lab is the following:
  \describe{
  \item{\bold{gate1:bidcut:}}{ Forward Scatter VS Side Scatter}
  \item{single gate }{(Select the lymphocytes--central cluster) }
  
  \item{\bold{gate2:bidcut:}}{ cd3 VS cd8}
  \item{gate 2.1:}{ (Select cd3+/cd8-)}
  \item{gate 2.2:}{ (Select cd3+/cd8+)}
  
  
  \item{\bold{gate3:biscut:}}{ cd69 vs Interferon Gamma}
  \item{gate 3.1:}{ (Select +/+ which are the cd4+ cells (from gate2.1))}
  \item{gate 3.2:}{ (Select +/+ which are the cd8+ cells (from gate2.2))}
  }  

  In General, the types of Gating/Cutting that are used in this gating
  scheme are the following:
  \describe{
  \item{uniscut = univariate single cut}{
    (Selection of the positive/right half)}
  
  \item{biscut = bivariate single cut}{
    (Selection of the +/-, -/-. +/+, or -/+ quadrant)}
  
  \item{bidcut = bivariate double cut}{
    (Selection of the center rectangle
    that results)}
  }

}
\value{
 
  \item{FCS object}{ with the following slots:}
    \item{data}{A augmented dataframe with the added-on gating column variables/indices}
    \item{metadata}{a FCSmetadata object with the information about the
      gating column variables: \$PnR (gating range), \$PnN
      (gating variable's shortname/unused name in the data of the FCS
      object), \$PnS (gating variable's longname/used name), and other slot information}
}

\references{Julie McElrath, PhD}
\author{A.J. Rossini and J.Y. Wan}
\note{
  The "FHCRC" data from the \pkg{rfcdorig} package can be used for this sequential
  gating scheme.  
}

\section{WARNING }{This gating scheme is not standard, and there may
  have been changes to the gating scheme.  This gating scheme only
  serves as an example, which demonstrates the use of
  \code{\link{createGate}},\code{\link{icreateGate}}
  and \code{\link{"[[-methods"}} which extracts the metadata information
  (eg. in order to obatin information about a previous gating
  index/column variable} 

\seealso{\code{\link{createGate}}, \code{\link{icreateGate}},
  \code{\link{showgate.FCS}},
  \code{\link{VRC.HVTNFCS}}, \code{\link{plotvar.FCS}},
  \code{\link{"[-methods"}}, \code{\link{"[[-methods"}}}

\examples{

  if (require(rfcdmin)){

    data.there<-is.element("MC.053",objects())
    if ( ( sum(data.there) != length(data.there) )){
      ## obtaining the FCS objects from VRC data
      data(MC.053min)
    }

    if (interactive()==TRUE){
      par(mfrow=c(4,2))

      MC.053.FHCRC<-FHCRC.HVTNFCS(MC.053)
    }
  }

}

\keyword{iplot}


