\name{convertS3toS4}
\alias{convertS3toS4}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Converts S3 class FCS object to S4 class FCS object }
\description{
  This function will update any S3 class \code{FCS} object to S4 class. 
}
\usage{
convertS3toS4(S3file, myFCSobj.name = "", fileName = "")
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{S3file}{S3 Class FCS object location and filename}
  \item{myFCSobj.name}{ character string indicating the FCS object name }
  \item{fileName}{ character string indicating the file name of the
    binary raw FCS data, from which the
    FCS object originates and which is read by \code{read.FCS}}
}
\details{
  The FCS object is obtained as the result of \code{read.FCS} which has
  been currently updated to output FCS objects as class S4 instead of S3.
}
\value{
  A Class S4 \code{FCS} object with the following slots:
 \item{data}{matrix of the data, where the rows are the
    cells/observations and the columns are the different fluoroescence
    measurements} 
  
  \item{metadata}{of class \code{\link{FCSmetadata}} with the following slots:
   \describe{ 
    \item{mode}{the mode of the raw binary file}
    
    \item{size}{numeric value describing the total number of rows or
      observations/cells}
    
    \item{nparam}{numeric value describing the number of columns or
      parameters}
    
    \item{shortnames}{a vector of the short names of the column variables of the data}
    
    \item{longnames}{a vector of the long names of the column variables of the data}

    \item{paramranges}{the vector of corresponding ranges or maximum
      values for each column variable}
    
    \item{filename}{character string of the name of the raw data file from which the object originates}
    
    \item{objectname}{character string of the name of the FCS S4 object}
    
    \item{original}{Boolean value indicating whether the data is the original}
    
    \item{fcsinfo}{list of other parameters}
  }
 }
}
\author{A.J. Rossini and J.Y. Wan}

\seealso{\code{\link{read.FCS}}, \code{\link{FCS}}}
\examples{
  if (require(rfcdmin)){

     ## if previously read-in as S3 FCS object
     facscan256.fcs<- paste(system.file("fcs", package="rfcdmin"),
                              "facscan256.fcs",
                              sep="/")

     ## reading in the FCS files
     FCSobj.S3<-read.FCS(facscan256.fcs, UseS3=TRUE)

     ## convert to S4 FCS

     FCSobj.S4<- convertS3toS4(FCSobj.S3,
                               myFCSobj.name="FCSobj.S4",
                               fileName=facscan256.fcs)
  }
}
\keyword{classes}
\keyword{character}
