\name{read.series.FCS}
\alias{read.series.FCS}

\title{ Reading a serie of raw binary Flow Cytometry Standard (FCS) files }
\description{
  Reads a serie of raw Flow Cytometry Standard (FCS) files and outputs several "FCS" R
  object. 
}
\usage{
read.series.FCS(fcsfiles,path=NULL,ext=NULL,...)
}
\arguments{
  \item{fcsfiles}{names of the FCS files without any extension}
  \item{path}{a character vector of full path names; the default corresponds to the working directory \code{\link{getwd}}}
   \item{ext}{character string giving optional extension to be added to each file name}
  \item{...}{any other arguments are passed to \code{\link{read.FCS}}}
}

\details{
  This function read several FCS files by the means of the
  \code{\link{read.FCS}} function. Thus,this function can also checks if there are discrepancies between
  the data and the metadata in terms of range and size (MY.DEBUG=TRUE).  If there is,
  then the data is re-read with different fcs.byte.size (1,2,4,8) and
  fcs.signed (TRUE, FALSE) combinations until there is no discrepancy
  between the data and the metadata.  If there is still a discrepancy,
  then the routine is halted.  Note: For FCS version 3.0 files, only the
  range of the data is checked against what is stated in the metadata
  because FCS version 3.0 files have extra elements that are read into
  the data.
}

\value{
  No value is returned. However a series of "FCS" object are created on
  the current environment with names of the form filename. The files names are given by the elements of slides. Each object is
  composed of the same data and metadata return by the \code{\link{read.FCS}} function.
}

\author{
  N. Le Meur
}


\seealso{
  \code{\link{read.FCS}},
  \code{\link{summary}},
  \code{\link{print}},
  \code{\link{extractGatedData}},
  \code{\link{addParameter}},
  \code{\link{"[-methods"}},
  \code{\link{"[[-methods"}},
  \code{fcs.type}
  \code{\link[prada]{readCytoSet}}
}

\examples{
 if (require(rfcdmin)) {

 ##obtaining the location of the fcs files in the data
  pathFiles<-system.file("bccrc", package="rfcdmin")
  drugFiles<-dir(pathFiles)

 ## reading in the FCS files
  drugData<-read.series.FCS(drugFiles,path=pathFiles,MY.DEBUG=FALSE)
  }
}

\keyword{classes}
\keyword{character}
