## StopOnError or else continue through
print("ok1")
if ("StopOnError" %in% objects()){
  if (!is.logical(StopOnError)) {
    ## default is TRUE when StopOnError is not boolean
    StopOnError <- TRUE
  }    
} else {
  ## default is to stop when there is an error 
  StopOnError <- TRUE
}

print("ok2")

## Create example and then test:
## An example FCS S4 object
if (!require(rfcdmin)){
  library(rfcdmin, lib.loc="~/lib/R")
}
if (!require(rflowcyt)){
  library(rflowcyt, lib.loc="~/lib/R")
}

print("ok3")

## Create S4 and S3 datasets that are the same.

ex.s4 <- orig.ex.s4 <- read.FCS(paste(system.file(package="rfcdmin"),
                        "/fcs/SEB-NP22.fcs", sep=""),
                  FCSobj.name="ex.s4", UseS3=FALSE)
ex.s3 <- read.FCS(paste(system.file(package="rfcdmin"),
                        "/fcs/SEB-NP22.fcs", sep=""),
                  FCSobj.name="ex.s4", UseS3=TRUE)
ErrorMsg <- NULL
FailedTestObjName <- NULL
error.printed <- names.failed <- NULL


## 2/12/2004
## JYW: Need the StopOnError flags and failed object name vector

## EXAMPLE FORMAT:
## create situation
## ex.s4 <- new("FCS")
## ex.s4@metadata[["$P1R"]]<-"test"

## if situation is unexpected, fail, else succeed
## if (ex.s4@metadata[["$P1R"]] != "test") {
##  stop("Test #1 failed: Can't set parameter\n")
##} else {
##  cat("Test #1 succeeded.\n")
##}


## Methods include:
## 1.  as.matrix(FCS)
## 2.  as.data.frame(FCS)
## 3.  as.FCS(matrix)
## 4.  as.FCS(data.frame)
## 5.  dim.FCS(FCS)
## 6.  show  (
## 7.  print
## 8.  plot
## 8.1.  summary
## ##  initialize
## 9. metaData
## 10. fluors
## 11. [  extracts data
## 12. [ <- replaces data
## 13. [[ extracts metadata
## 14. [[ <- replaces the metadata

## 15. addParameter
## 16. checkvars
## 17. fixvars
## 18. equals
## Examples corresponding to above methods:
t1<-as(ex.s4, "matrix")
if (is(t1, "matrix") != TRUE) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t1 failed: Can't set 'FCS' as 'matrix' class")
  FailedTestObjName <- c(FailedTestObjName, "t1")
  if (StopOnError) {
    stop("t1 failed: Can't set 'FCS' as 'matrix' class\n")
  }
} else {
  cat("t1 succeeded: Able to set 'FCS' as 'matrix' class\n")
}

t2<-as(ex.s4, "data.frame")
if (is(t2, "data.frame") != TRUE) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t2 failed: Can't set 'FCS' as 'data.frame' class")
  FailedTestObjName <- c(FailedTestObjName, "t2")
  if (StopOnError){
    stop("t2 failed: Can't set 'FCS' as 'data.frame' class\n")
  }
} else {
  cat("t2 succeeded: Able to set 'FCS' as 'data.frame' class\n")
}

t3<-as(matrix(c(0,0,0,0), ncol=2), "FCS")
if (is(t3, "FCS") != TRUE) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t3 failed: Can't set 'matrix' as 'FCS' class")
  FailedTestObjName <- c(FailedTestObjName, "t3")
  if (StopOnError){
    stop("t3 failed: Can't set 'matrix' as 'FCS' class\n")
  }
} else {
  cat("t3 succeeded: Able to set 'matrix' as 'FCS' class.\n")
}


t4<-as(data.frame(cbind(rep(1,3), rep(2, 3))), "FCS")
if (is(t4, "FCS") != TRUE) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t4 failed: Can't set 'data.frame' as 'FCS' class")
  FailedTestObjName <- c(FailedTestObjName, "t4")
  if (StopOnError){
    stop("t4 failed: Can't set 'data.frame' as 'FCS' class\n")
  }
} else {
  cat("t4 succeeded: Able to set 'data.frame' as 'FCS' class.\n")
}

t5 <- dim.FCS(ex.s4)
if (sum(t5==dim(ex.s4@data))!=2) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t5 failed: Can't obtain dimensions of the 'FCS' data by using 'dim.FCS'")
  FailedTestObjName <- c(FailedTestObjName, "t5")
  if (StopOnError){
    stop("t5 failed: Can't obtain dimensions of the 'FCS' data by using 'dim.FCS'\n")
  }
} else {
  cat("t5 succeeded: Able to obtain dimensions of the 'FCS' data by using 'dim.FCS'.\n")
}


## t6
show(ex.s4)
## t7
print(ex.s4)

## t8
## plot(ex.s4)  ## does work, but takes a LONG time

## t8.1



t8.1 <- summary(ex.s4) ## does work, FIXED 3.23.04
## and does not need to test


t9 <- metaData(ex.s4)
if ((is(t9, "FCSmetadata") != TRUE)){
  ErrorMsg <- c(ErrorMsg, "FCSclass: t9 failed: Can't obtain 'FCS' metadata by using 'metaData'")
  FailedTestObjName <- c(FailedTestObjName, "t9")
  if (StopOnError){
    stop("t9 failed: Can't obtain 'FCS' metadata by using 'metaData'\n")
  }
} else {
  cat("t9 succeeded: Able to obtain 'FCS' metadata by using 'metaData' \n")
}

t10 <- fluors(ex.s4)
if ((is(t10, "matrix") != TRUE) || (sum(t10 == ex.s4@data)!= length(unlist(ex.s4@data)))) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t10 failed: Can't obtain 'FCS' data by using 'fluors'")
  FailedTestObjName <- c(FailedTestObjName, "t10")
  if (StopOnError){
    stop("t10 failed: Can't obtain 'FCS' data by using 'fluors'\n")
  }
} else {
  cat("t10 succeeded: Able to obtain 'FCS' data by using 'fluors' \n")
}

### Testing [...] extraction of the FCS data
t11.1<-ex.s4[1,3]
t11.1 <- as(t11.1, "matrix")
if (sum(t11.1 == ex.s4@data[1,3])!= length(unlist(ex.s4@data[1,3]))) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t11.1 failed: Can't extract 'FCS' data[1,3] by using [...] on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t11.1")
  if (StopOnError){
    stop("t11.1 failed: Can't extract 'FCS' data[1,3] by using [...] on 'FCS' object\n")
  }
} else {
  cat("t11.1 succeeded: Able to extract 'FCS' data[1,3] by using [...] on FCS object\n")
}


t11.2<-as(ex.s4[1,], "matrix")
if (sum(t11.2 == ex.s4@data[1,])!= length(unlist(ex.s4@data[1,]))) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t11.2 failed: Can't extract 'FCS' data 1st row by using [1,] on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t11.2")
  if (StopOnError){
    stop("t11.2 failed: Can't extract 'FCS' data 1st row by using [1,] on 'FCS' object\n")
  }
} else {
  cat("t11.2 succeeded: Able to extract 'FCS' data 1st row by using [1,] on FCS object\n")
}

t11.3<-as(ex.s4[1:10,], "matrix")
if (sum(t11.3 == ex.s4@data[1:10,])!= length(unlist(ex.s4@data[1:10,]))) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t11.3 failed: Can't extract 'FCS' data 1st 10 rows by using [1:10,] on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t11.3")
  if (StopOnError){
    stop("t11.3 failed: Can't extract 'FCS' data's 1st 10 rows by using [1:10,] on 'FCS' object\n")
  }
} else {
  cat("t11.3 succeeded: Able to extract 'FCS' data's 1st 10 rows by using [1:10,] on FCS object\n")
}


t11.4 <- as(ex.s4[,2], "matrix")
if (sum(t11.4 == ex.s4@data[,2])!= length(unlist(ex.s4@data[,2]))) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t11.4 failed: Can't extract 'FCS' data's 2nd column by using [,2] on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t11.4")
  if (StopOnError){
    stop("t11.4 failed: Can't extract 'FCS' data's 2nd column by using [,2] on 'FCS' object\n")
  }
} else {
  cat("t11.4 succeeded: Able to extract 'FCS' data's 2nd column by using [,2] on FCS object\n")
}


t11.5 <- as(ex.s4[,2:10], "matrix")
if (sum(t11.5 == ex.s4@data[,2:10])!= length(unlist(ex.s4@data[,2:10]))) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t11.5 failed: Can't extract 'FCS' data's 2nd-10th columns by using [,2:10] on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t11.5")
  if (StopOnError){
    stop("t11.5 failed: Can't extract 'FCS' data's 2nd-10th columns by using [,2:10] on 'FCS' object\n")
  }
} else {
  cat("t11.5 succeeded: Able to extract 'FCS' data's 2nd-10th columns by using [,2:10] on FCS object\n")
}


t11.6 <- as(ex.s4[3:9, 2:4], "matrix")
if (sum(t11.6 == ex.s4@data[3:9,2:4])!= length(unlist(ex.s4@data[3:9,2:4]))) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t11.6 failed: Can't extract 'FCS' data[3:9,2:4] by using [...] on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t11.6")
  if (StopOnError){
    stop("t11.6 failed: Can't extract 'FCS' data[3:9,2:4] by using [...] on 'FCS' object\n")
  }
} else {
  cat("t11.6 succeeded: Able to extract 'FCS' data[3:9,2:4] by using [...] on FCS object\n")
}

### Testing of the [ <- replace method for the FCS data

ex.s4[1,3]<- t12.1 <- -1234567
if (t12.1 != ex.s4@data[1,3]) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t12.1 failed: Can't replace 'FCS' data[1,3] by using [...]<- on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t12.1")
  if (StopOnError){
    stop("t12.1 failed: Can't replace 'FCS' data[1,3] by using [...]<- on 'FCS' object\n")
  }
} else {
  cat("t12.1 succeeded: Able to replace 'FCS' data[1,3] by using [...]<- on FCS object\n")
}

ex.s4[1,] <- t12.2<-rep(-1, ex.s4[["nparam"]])
if (sum(t12.2 == ex.s4@data[1,])!=length(ex.s4@data[1,])) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t12.2 failed: Can't replace 'FCS' data[1,] by using [...]<- on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t12.2")
  if (StopOnError){
    stop("t12.2 failed: Can't replace 'FCS' data[1,] by using [...]<- on 'FCS' object\n")
  }
} else {
  cat("t12.2 succeeded: Able to replace 'FCS' data[1,] by using [...]<- on FCS object\n")
}

ex.s4[1:10,] <- t12.3<-matrix(1:(10*ex.s4[["nparam"]]), nrow=10)
if (sum(t12.3 == ex.s4@data[1:10,])!=length(ex.s4@data[1:10,])) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t12.3 failed: Can't replace 'FCS' data[1:10,] by using [...]<- on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t12.3")
  if (StopOnError){
    stop("t12.3 failed: Can't replace 'FCS' data[1:10,] by using [...]<- on 'FCS' object\n")
  }
} else {
  cat("t12.3 succeeded: Able to replace 'FCS' data[1:10,] by using [...]<- on FCS object\n")
}

t12.4 <- ex.s4[,2] <- matrix(1:ex.s4@metadata@size,ncol=1) 
if (sum(t12.4 == ex.s4@data[,2])!= length(unlist(ex.s4@data[,2]))) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t12.4 failed: Can't replace 'FCS' data's 2nd column by using [,2]<- on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t12.4")
  if (StopOnError){
    stop("t12.4 failed: Can't replace 'FCS' data's 2nd column by using [,2]<- on 'FCS' object\n")
  }
} else {
  cat("t12.4 succeeded: Able to replace 'FCS' data's 2nd column by using [,2]<- on FCS object\n")
}


t12.5 <- ex.s4[,2:10] <- matrix(rep(1:ex.s4@metadata@size, 9), ncol=9)
if (sum(t12.5 == ex.s4@data[,2:10])!= length(unlist(ex.s4@data[,2:10]))) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t12.5 failed: Can't replace 'FCS' data[,2:10] by using [...]<- on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t12.5")
  if (StopOnError){
    stop("t12.5 failed: Can't replace 'FCS' data's 2nd-10th columns by using [,2:10]<- on 'FCS' object\n")
  }
} else {
  cat("t12.5 succeeded: Able to replace 'FCS' data's 2nd-10th columns by using [,2:10]<- on FCS object\n")
}


t12.6 <- ex.s4[3:9, 2:4] <- matrix(rep(1:7, 3), ncol=3)
if (sum(t12.6 == ex.s4@data[3:9,2:4])!= length(unlist(ex.s4@data[3:9,2:4]))) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t12.6 failed: Can't replace 'FCS' data[3:9, 2:4] by using [...]<- on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t12.6")
  if (StopOnError){
    stop("t12.6 failed: Can't replace 'FCS' data[3:9,2:4] by using [...]<- on 'FCS' object\n")
  }
} else {
  cat("t12.6 succeeded: Able to replace 'FCS' data[3:9,2:4] by using [...]<- on FCS object\n")
}
  
## changing back to the original data
t12.1.1 <- t11.1
ex.s4[1,3] <- t12.1.1

if (sum(t12.1.1 == ex.s4@data[1,3])!= length(unlist(ex.s4@data[1,3]))) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t12.1.1 failed: Can't replace 'FCS' data[1,3] by using [...]<- on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t12.1.1")
  if (StopOnError){
    stop("t12.1.1 failed: Can't replace 'FCS' data[1,3] by using [...]<- on 'FCS' object\n")
  }
} else {
  cat("t12.1.1 succeeded: Able to replace 'FCS' data[1,3] by using [...]<- on FCS object\n")
}
  
  
 
ex.s4[1,] <- t12.1.2 <- t11.2
if (sum(t12.1.2 == ex.s4@data[1,])!= length(unlist(ex.s4@data[1,]))) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t12.1.2 failed: Can't replace 'FCS' data[1,] by using [...]<- on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t12.1.2")
  if (StopOnError){
    stop("t12.1.2 failed: Can't replace 'FCS' data 1st row by using [1,]<- on 'FCS' object\n")
  }
} else {
  cat("t12.1.2 succeeded: Able to replace 'FCS' data 1st row by using [1,]<- on FCS object\n")
}

ex.s4[1:10,] <- t12.1.3 <- t11.3
if (sum(t12.1.3 == ex.s4@data[1:10,])!= length(unlist(ex.s4@data[1:10,]))) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t12.1.3 failed: Can't replace 'FCS' data[1:10,] by using [...]<- on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t12.1.3")
  if (StopOnError){
    stop("t12.1.3 failed: Can't replace 'FCS' data's 1st 10 rows by using [1:10,]<- on 'FCS' object\n")
  }
} else {
  cat("t12.1.3 succeeded: Able to replace 'FCS' data's 1st 10 rows by using [1:10,]<- on FCS object\n")
}


ex.s4[,2] <- t12.1.4 <- t11.4 
if (sum(t12.1.4 == ex.s4@data[,2])!= length(unlist(ex.s4@data[,2]))) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t12.1.4 failed: Can't replace 'FCS' data[,2] by using [...]<- on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t12.1.4")
  if (StopOnError){
    stop("t12.1.4 failed: Can't replace 'FCS' data's 2nd column by using [,2]<- on 'FCS' object\n")
  }
} else {
  cat("t12.1.4 succeeded: Able to replace 'FCS' data's 2nd column by using [,2]<- on FCS object\n")
}


ex.s4[,2:10] <- t12.1.5 <- t11.5 
if (sum(t12.1.5 == ex.s4@data[,2:10])!= length(unlist(ex.s4@data[,2:10]))) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t12.1.5 failed: Can't replace 'FCS' data[,2:10] by using [...]<- on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t12.1.5")
  if (StopOnError){
    stop("t12.1.5 failed: Can't replace 'FCS' data's 2nd-10th columns by using [,2:10]<- on 'FCS' object\n")
  }
} else {
  cat("t12.1.5 succeeded: Able to replace 'FCS' data's 2nd-10th columns by using [,2:10]<- on FCS object\n")
}


ex.s4[3:9, 2:4] <- t12.1.6 <- t11.6 
if (sum(t12.1.6 == ex.s4@data[3:9,2:4])!= length(unlist(ex.s4@data[3:9,2:4]))) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t12.1.6 failed: Can't replace 'FCS' data[3:9,2:4] by using [...]<- on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t12.1.6")
  if (StopOnError){
    stop("t12.1.6 failed: Can't replace 'FCS' data[3:9,2:4] by using [...]<- on 'FCS' object\n")
  }
} else {
  cat("t12.1.6 succeeded: Able to replace 'FCS' data[3:9,2:4] by using [...]<- on FCS object\n")
}
###  What about having NA or NULL??
###  cannot replace with a NULL value
t12.1.7 <- ex.s4[3,2]
ex.s4[3, 2] <-NA
if (!is.na(as(ex.s4[3,2], "matrix"))) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t12.1.7a failed: Can't replace 'FCS' data[3,2] by using [...]<-NA on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t12.1.7a")
  if (StopOnError){
    stop("t12.1.7a failed: Can't replace 'FCS' data[3,2] by using [...]<- NA on 'FCS' object\n")
  }
} else {
  cat("t12.1.7a succeeded: Able to replace 'FCS' data[3,2] by using [...]<- NA on FCS object\n")
}
ex.s4[3,2] <- t12.1.7 <- as(t12.1.7, "matrix")

if (sum(t12.1.7 == as(ex.s4@data[3,2], "matrix"))!= length(unlist(ex.s4@data[3,2]))) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t12.1.7b failed: Can't replace 'FCS' data[3,2] by using [...]<- on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t12.1.7b")
  if (StopOnError){
    stop("t12.1.7b failed: Can't replace 'FCS' data[3,2] by using [...]<-  on 'FCS' object\n")
  }
} else {
  cat("t12.1.7b succeeded: Able to replace 'FCS' data[3,2] by using [...]<-  on FCS object\n")
}

## 13: [[

t13.1 <- ex.s4[["$P1R"]]
if (sum(t13.1 == ex.s4@metadata[["$P1R"]])!= length(unlist(ex.s4@metadata["$P1R"]))) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t13.1 failed: Can't extract 'FCS' metadata[['$P1R']] by using [[...]] on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t13.1")
  if (StopOnError){
    stop("t13.1 failed: Can't extract 'FCS' metadata[['$P1R']] by using [[...]]  on 'FCS' object\n")
  }
} else {
  cat("t13.1 succeeded: Able to extract 'FCS' metadata[['$P1R']] by using [[...]] on FCS object\n")
}

t13.2 <- ex.s4[["$MODE"]]
if (sum(t13.2 == ex.s4@metadata[["$MODE"]])!= length(unlist(ex.s4@metadata["$MODE"]))) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t13.2 failed: Can't extract 'FCS' metadata[['$P1R']] by using [[...]] on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t13.2")
  if (StopOnError){
    stop("t13.2 failed: Can't extract 'FCS' metadata[['$MODE']] by using [[...]]  on 'FCS' object\n")
  }
} else {
  cat("t13.2 succeeded: Able to extract 'FCS' metadata[['$MODE']] by using [[...]] on FCS object\n")
}
t13.3 <- ex.s4[["$PnS"]]
if (sum(t13.3[!is.na(t13.3)] == ex.s4@metadata[["$PnS"]][!is.na( ex.s4@metadata[["$PnS"]])])!= length(unlist(ex.s4@metadata["$PnS"][!is.na( ex.s4@metadata[["$PnS"]])]))) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t13.3 failed: Can't extract 'FCS' metadata[['$PnS']] by using [[...]] on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t13.3")
  if (StopOnError){
    stop("t13.3 failed: Can't extract 'FCS' metadata[['$PnS']] by using [[...]]  on 'FCS' object\n")
  }
} else {
  cat("t13.3 succeeded: Able to extract 'FCS' metadata[['$PnS']] by using [[...]] on FCS object\n")
}
t13.4 <- ex.s4[[1:10]]
if (sum(unlist(t13.4) == unlist(ex.s4@metadata[[1:10]]))!= length(unlist(ex.s4@metadata[1:10]))) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t13.4 failed: Can't extract 'FCS' metadata[[1:10]] by using [[...]] on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t13.4")
  if (StopOnError){
    stop("t13.4 failed: Can't extract 'FCS' metadata[[1:10]] by using [[...]]  on 'FCS' object\n")
  }
} else {
  cat("t13.4 succeeded: Able to extract 'FCS' metadata[[1:10]] by using [[...]] on FCS object\n")
}
t13.5 <- ex.s4[["size"]]
if (sum(t13.5 == ex.s4@metadata[["size"]])!= length(unlist(ex.s4@metadata["size"]))) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t13.5 failed: Can't extract 'FCS' metadata[['size']] by using [[...]] on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t13.5")
  if (StopOnError){
    stop("t13.5 failed: Can't extract 'FCS' metadata[['size']] by using [[...]]  on 'FCS' object\n")
  }
} else {
  cat("t13.5 succeeded: Able to extract 'FCS' metadata[['size']] by using [[...]] on FCS object\n")
}

t13.6 <- ex.s4[["$PnN"]]
if (sum(t13.6 == ex.s4@metadata[["$PnN"]]) != length(unlist(ex.s4@metadata["$PnN"]))) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t13.6 failed: Can't extract 'FCS' metadata[['$PnN']] by using [[...]] on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t13.6")
  if (StopOnError){
    stop("t13.6 failed: Can't extract 'FCS' metadata[['$PnN']] by using [[...]]  on 'FCS' object\n")
  }
} else {
  cat("t13.6 succeeded: Able to extract 'FCS' metadata[['$PnN']] by using [[...]] on FCS object\n")
}

t13.7 <- ex.s4[["$P3N"]]
if (sum(t13.7 == ex.s4@metadata[["$P3N"]]) != length(unlist(ex.s4@metadata["$P3N"]))) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t13.7 failed: Can't extract 'FCS' metadata[['$P3N']] by using [[...]] on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t13.7")
  if (StopOnError){
    stop("t13.7 failed: Can't extract 'FCS' metadata[['$P3N']] by using [[...]]  on 'FCS' object\n")
  }
} else {
  cat("t13.7 succeeded: Able to extract 'FCS' metadata[['$P3N']] by using [[...]] on FCS object\n")
}

t13.8 <- ex.s4[["$TOT"]]
if (sum(t13.8 == ex.s4@metadata[["size"]])!= length(unlist(ex.s4@metadata["$TOT"]))) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t13.8 failed: Can't extract 'FCS' metadata[['$TOT']] by using [[...]] on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t13.8")
  if (StopOnError){
    stop("t13.8 failed: Can't extract 'FCS' metadata[['$TOT']] by using [[...]]  on 'FCS' object\n")
  }
} else {
  cat("t13.8 succeeded: Able to extract 'FCS' metadata[['$TOT']] by using [[...]] on FCS object\n")
}
t13.9 <- ex.s4[[1:4]]
if (sum(unlist(t13.9) == unlist(ex.s4@metadata[[1:4]]))!= length(unlist(ex.s4@metadata[1:4]))) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t13.9 failed: Can't extract 'FCS' metadata[[1:4]] by using [[...]] on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t13.9")
  if (StopOnError){
    stop("t13.9 failed: Can't extract 'FCS' metadata[[1:4]] by using [[...]]  on 'FCS' object\n")
  }
} else {
  cat("t13.9 succeeded: Able to extract 'FCS' metadata[[1:4]] by using [[...]] on FCS object\n")
}


t13.10 <- ex.s4[[4]]
if (!identical(t13.10, ex.s4@metadata[[4]])) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t13.10 failed: Can't extract 'FCS' metadata[[4]] by using [[...]] on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t13.10")
  if (StopOnError){
    stop("t13.10 failed: Can't extract 'FCS' metadata[[4]] by using [[...]]  on 'FCS' object\n")
  }
} else {
  cat("t13.10 succeeded: Able to extract 'FCS' metadata[[4]] by using [[...]] on FCS object\n")
}
## not in the metadata
## if character name not there then null value is returned
t13.11 <- ex.s4[["notthere"]]
if (!is.null(t13.11)) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t13.11 failed: Can't extract 'FCS' metadata[['notthere']] by using [[...]] on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t13.11")
  if (StopOnError){
    stop("t13.11 failed: Can't extract 'FCS' metadata[['notthere']] by using [[...]]  on 'FCS' object\n")
  }
} else {
  cat("t13.11 succeeded: Able to extract 'FCS' metadata[['notthere']] by using [[...]] on FCS object\n")
}

## if numeric index is not there, then an empty list is returned
t13.12 <- ex.s4[[100000]]
if (length(t13.12)!=0) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t13.12 failed: Can't extract 'FCS' metadata[[100000]] by using [[...]] on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t13.12")
  if (StopOnError){
    stop("t13.12 failed: Can't extract 'FCS' metadata[[100000]] by using [[...]]  on 'FCS' object\n")
  }
} else {
  cat("t13.12 succeeded: Able to extract 'FCS' metadata[[100000]] by using [[...]] on FCS object\n")
}
## 14: [[<-
## can make a new slot?  by name only.. not by numeric index
ex.s4[["newslot"]] <- t14.0 <- "yo man"

if (sum(t14.0 == ex.s4@metadata@fcsinfo[["newslot"]])!= length(unlist(ex.s4@metadata@fcsinfo["newslot"]))) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t14.0 failed: Can't set 'FCS' metadata[[newslot]] by using [[...]]<- on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t14.0")
  if (StopOnError){
    stop("t14.0 failed: Can't set 'FCS' metadata[[newslot]] by using [[...]]<-  on 'FCS' object\n")
  }
} else {
  cat("t14.0 succeeded: Able to set 'FCS' metadata[[newslot]] by using [[...]]<- on FCS object\n")
}


t14.1 <- ex.s4[["$P1R"]] <- "-1"
if (sum(t14.1 == ex.s4@metadata[["$P1R"]])!= length(unlist(ex.s4@metadata["$P1R"]))) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t14.1 failed: Can't replace 'FCS' metadata[['$P1R']] by using [[...]]<- on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t14.1")
  if (StopOnError){
    stop("t14.1 failed: Can't replace 'FCS' metadata[['$P1R']] by using [[...]]<-  on 'FCS' object\n")
  }
} else {
  cat("t14.1 succeeded: Able to replace 'FCS' metadata[['$P1R']] by using [[...]]<- on FCS object\n")
}

t14.2 <- ex.s4[["$MODE"]] <- "Q"
if (sum(t14.2 == ex.s4@metadata[["$MODE"]])!= length(unlist(ex.s4@metadata["$MODE"]))) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t14.2 failed: Can't replace 'FCS' metadata[['$MODE']] by using [[...]]<- on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t14.2")
  if (StopOnError){
    stop("t14.2 failed: Can't replace 'FCS' metadata[['$MODE']] by using [[...]]<-  on 'FCS' object\n")
  }
} else {
  cat("t14.2 succeeded: Able to replace 'FCS' metadata[['$MODE']] by using [[...]]<- on FCS object\n")
}

t14.3 <- ex.s4[["$PnS"]] <- rep("none", ex.s4[["nparam"]])
if (sum(t14.3[!is.na(t14.3)] == ex.s4@metadata[["$PnS"]][!is.na( ex.s4@metadata[["$PnS"]])])!= length(unlist(ex.s4@metadata["$PnS"][!is.na( ex.s4@metadata[["$PnS"]])]))) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t14.3 failed: Can't replace 'FCS' metadata[['$PnS']] by using [[...]]<- on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t14.3")
  if (StopOnError){
    stop("t14.3 failed: Can't replace 'FCS' metadata[['$PnS']] by using [[...]]<-  on 'FCS' object\n")
  }
} else {
  cat("t14.3 succeeded: Able to replace 'FCS' metadata[['$PnS']] by using [[...]]<- on FCS object\n")
}
t14.4 <- ex.s4[[1:10]] <- list(1:10)
if (sum(unlist(t14.4) == unlist(ex.s4@metadata[[1:10]]))!= length(unlist(ex.s4@metadata[1:10]))) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t14.4 failed: Can't replace 'FCS' metadata[[1:10]] by using [[...]]<- on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t14.4")
  if (StopOnError){
    stop("t14.4 failed: Can't replace 'FCS' metadata[[1:10]] by using [[...]]<-  on 'FCS' object\n")
  }
} else {
  cat("t14.4 succeeded: Able to replace 'FCS' metadata[[1:10]] by using [[...]]<- on FCS object\n")
}
t14.5 <- ex.s4[["size"]] <- 0
if (sum(t14.5 == ex.s4@metadata[["size"]])!= length(unlist(ex.s4@metadata["size"]))) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t14.5 failed: Can't replace 'FCS' metadata[['size']] by using [[...]]<- on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t14.5")
  if (StopOnError){
    stop("t14.5 failed: Can't replace 'FCS' metadata[['size']] by using [[...]]<-  on 'FCS' object\n")
  }
} else {
  cat("t14.5 succeeded: Able to replace 'FCS' metadata[['size']] by using [[...]]<- on FCS object\n")
}
t14.6 <- ex.s4[["$PnN"]] <- rep("none", ex.s4[["nparam"]])
if (sum(t14.6 == ex.s4@metadata[["$PnN"]])!= length(unlist(ex.s4@metadata["$PnN"]))) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t14.6 failed: Can't replace 'FCS' metadata[['$PnN']] by using [[...]]<- on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t14.6")
  if (StopOnError){
    stop("t14.6 failed: Can't replace 'FCS' metadata[['$PnN']] by using [[...]]<-  on 'FCS' object\n")
  }
} else {
  cat("t14.6 succeeded: Able to replace 'FCS' metadata[['$PnN']] by using [[...]]<- on FCS object\n")
}
t14.7 <- ex.s4[["$P3N"]] <- "pulpfiction"
if (sum(t14.7 == ex.s4@metadata[["$P3N"]])!= length(unlist(ex.s4@metadata["$P3N"]))) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t14.7 failed: Can't replace 'FCS' metadata[['$P3N']] by using [[...]]<- on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t14.7")
  if (StopOnError){
    stop("t14.7 failed: Can't replace 'FCS' metadata[['$P3N']] by using [[...]]<-  on 'FCS' object\n")
  }
} else {
  cat("t14.7 succeeded: Able to replace 'FCS' metadata[['$P3N']] by using [[...]]<- on FCS object\n")
}
t14.8 <- ex.s4[["$TOT"]] <- 2
if (sum(t14.8 == ex.s4@metadata[["size"]])!= length(unlist(ex.s4@metadata["$TOT"]))) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t14.8 failed: Can't replace 'FCS' metadata[['$TOT']] by using [[...]]<- on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t14.8")
  if (StopOnError){
    stop("t14.8 failed: Can't replace 'FCS' metadata[['$TOT']] by using [[...]]<-  on 'FCS' object\n")
  }
} else {
  cat("t14.8 succeeded: Able to replace 'FCS' metadata[['$TOT']] by using [[...]]<- on FCS object\n")
}
t14.9 <- ex.s4[[1:4]] <- list(1:4)
if (sum(unlist(t14.9) == unlist(ex.s4@metadata[[1:4]]))!= length(unlist(ex.s4@metadata[1:4]))) {
  ErrorMsg <- c(ErrorMsg, "FCSclass: t14.9 failed: Can't replace 'FCS' metadata[[1:4]] by using [[...]]<- on 'FCS' object")
  FailedTestObjName <- c(FailedTestObjName, "t14.9")
  if (StopOnError){
    stop("t14.9 failed: Can't replace 'FCS' metadata[[1:4]] by using [[...]]<-  on 'FCS' object\n")
  }
} else {
  cat("t14.9 succeeded: Able to replace 'FCS' metadata[[1:4]] by using [[...]]<- on FCS object\n")
}
t14.10 <- ex.s4[[4]] <- "taxicab"
  if (sum(t14.10 == ex.s4@metadata[[4]])!= length(unlist(ex.s4@metadata[4]))) {
    ErrorMsg <- c(ErrorMsg, "FCSclass: t14.10 failed: Can't replace 'FCS' metadata[[4]] by using [[...]]<- on 'FCS' object")
    FailedTestObjName <- c(FailedTestObjName, "t14.10")
  if (StopOnError){
    stop("t14.10 failed: Can't replace 'FCS' metadata[[4]] by using [[...]]<-  on 'FCS' object\n")
  }
} else {
  cat("t14.10 succeeded: Able to replace 'FCS' metadata[[4]] by using [[...]]<- on FCS object\n")
}


## 15. addParameter
t15 <- addParameter(orig.ex.s4, 1:(dim.FCS(orig.ex.s4)[1]), shortname="t15", longname="t15.long")
if ((dim.FCS(t15)[2])!=((dim.FCS(orig.ex.s4)[2]) + 1)){
  ErrorMsg <- c(ErrorMsg, "FCSclass: t15 failed: addParameter failed on FCS object")
  FailedTestObjName <- c(FailedTestObjName, "t15")
  if (StopOnError){
    stop("t15 failed: addParameter failed on FCS object\n")
  }
} else {
  cat("t15 succeeded: addParameter succeeded on FCS object\n")
}
## 16:checkvars

###  JYW: There is an error here  2/9/04
t16.1 <- checkvars(ex.s4)
if (t16.1 !=FALSE){
  ErrorMsg <- c(ErrorMsg, "FCSclass: t16.1 failed: checkvars failed on FCS object")
  FailedTestObjName <- c(FailedTestObjName, "t16.1")
  if (StopOnError){
    stop("t16.1 failed: checkvars failed on FCS object\n")
  }
} else {
  cat("t16.1 succeeded: checkvars succeeded on FCS object\n")
}
t16.2 <- checkvars(orig.ex.s4)
if (t16.2 !=TRUE){
  ErrorMsg <- c(ErrorMsg, "FCSclass: t16.2 failed: checkvars failed on FCS object")
  FailedTestObjName <- c(FailedTestObjName, "t16.2")
  if (StopOnError){
    stop("t16.2 failed: checkvars failed on FCS object\n")
  }
} else {
  cat("t16.2 succeeded: checkvars succeeded on FCS object\n")
}

t16.3 <- checkvars(t15)
if (t16.3 !=TRUE){
  ErrorMsg <- c(ErrorMsg, "FCSclass: t16.3 failed: checkvars failed on FCS object")
  FailedTestObjName <- c(FailedTestObjName, "t16.3")
  if (StopOnError){
    stop("t16.3 failed: checkvars failed on FCS object\n")
  }
} else {
  cat("t16.3 succeeded: checkvars succeeded on FCS object\n")
}

## 17. fixvars


t17.1 <- fixvars(ex.s4)

if (equals(t17.1, ex.s4)){
  ErrorMsg <- c(ErrorMsg, "FCSclass: t17.1 failed: fixvars failed on FCS object")
  FailedTestObjName <- c(FailedTestObjName, "t17.1")
  if (StopOnError){
    stop("t17.1 failed: fixvars failed on FCS object\n")
  }
} else {
  cat("t17.1 succeeded: fixvars succeeded on FCS object\n")
}

t17.2 <- fixvars(t15)

if (!equals(t17.2, t15)){
  ErrorMsg <- c(ErrorMsg, "FCSclass: t17.2 failed: fixvars failed on FCS object")
  FailedTestObjName <- c(FailedTestObjName, "t17.2")
  if (StopOnError){
    stop("t17.2 failed: fixvars failed on FCS object\n")
  }
} else {
  cat("t17.2 succeeded: fixvars succeeded on FCS object\n")
}
## equals
t18 <- !equals(orig.ex.s4, ex.s4)
if (!t18){
  ErrorMsg <- c(ErrorMsg, "FCSclass: t18 failed: equals failed on FCS object")
  FailedTestObjName <- c(FailedTestObjName, "t18")
  if (StopOnError){
    stop("t18 failed: equals failed on FCS object\n")
  }
} else {
  cat("t18 succeeded: equals succeeded on FCS object\n")
}
## must be able to check against data range that is NA, (work up example)
## RFACSadd>>$P5R, etc...  chekc it

if (is.null(FailedTestObjName) & is.null(ErrorMsg)){
  names.failed <- "None; all FCSmetadataclass tests were successful."
  error.printed <- "None; all FCSmetadataclass tests were successful."
} else {
  names.failed <- matrix(FailedTestObjName, ncol=1)
  error.printed <- ErrorMsg
}
cat("These are the following test objects that have failed: \n\n")
print(names.failed)
cat("\n\n\n")
cat("These are the corresponding Error Messages: \n\n")
print(error.printed)
